@@ -1,8 +1,6 @@
 import os
 import paramiko
 from plumbum import colors
-from PyQt5.QtWidgets import QSlider, QCheckBox, QSpinBox, QDoubleSpinBox, \
-    QTabWidget, QRadioButton, QComboBox
 
 import linien
 from linien.config import REMOTE_BASE_PATH
@@ -70,7 +68,7 @@ def read_remote_version(ssh):
 
 def upload_source_code(ssh):
     """Uploads the application's source code to the remote server using SFTP."""
-    print('uploading dev source code..')
+    print('uploading dev source code...')
 
     ftp = ssh.open_sftp()
 
@@ -81,6 +79,12 @@ def upload_source_code(ssh):
         )
     )
 
+    if not os.path.exists(os.path.join(directory, 'server', 'linien.bin')):
+        print(colors.red | 'Error: In order to run the development version, '
+                           'you need the FPGA bitstream in server/linien.bin! '
+                           'Consult README to see how you can get one.')
+        raise Exception('FPGA bitstream missing')
+
     # upload the code required for running the server
     for dirpath, dirnames, filenames in os.walk(directory):
         dirpath_rel = dirpath.replace(directory, '').lstrip('/')
@@ -101,31 +105,3 @@ def upload_source_code(ssh):
             ftp.put(local_path, remote_filepath)
 
     ftp.close()
-
-
-def param2ui(parameter, element, process_value=lambda x: x):
-    """Updates ui elements according to parameter values.
-
-    Listens to parameter changes and sets the value of `element` automatically.
-    Optionally, the value can be processed using `process_value`.
-    This function should be used because it automatically blocks signal
-    emission from the target element; otherwise this can cause nasty
-    endless loops when quickly changing a paramater multiple times.
-    """
-    def on_change(value, element=element):
-        element.blockSignals(True)
-
-        value = process_value(value)
-
-        if isinstance(element, (QSlider, QSpinBox, QDoubleSpinBox)):
-            element.setValue(value)
-        elif isinstance(element, (QCheckBox, QRadioButton)):
-            element.setChecked(value)
-        elif isinstance(element, (QTabWidget, QComboBox)):
-            element.setCurrentIndex(int(value))
-        else:
-            raise Exception('unsupported element type %s' % type(element))
-
-        element.blockSignals(False)
-
-    parameter.change(on_change)
\ No newline at end of file
