@@ -1,9 +1,9 @@
 import pickle
 import traceback
 import numpy as np
-from time import sleep, time
-from linien.common import determine_shift_by_correlation, get_lock_point, \
-    control_signal_has_correct_amplitude, combine_error_signal
+from linien.common import get_lock_point, combine_error_signal, \
+    check_plot_data, ANALOG_OUT0, SpectrumUncorrelatedException
+from linien.server.approach_line import Approacher
 
 
 class Autolock:
@@ -14,20 +14,22 @@ class Autolock:
 
         self.first_error_signal = None
         self.parameters.autolock_running.value = False
+        self.parameters.autolock_retrying.value = False
+
         self.should_watch_lock = False
+        self.approacher = None
+        self._data_listener_added = False
 
         self.reset_properties()
 
     def reset_properties(self):
-        self.history = []
-        self.zoom_factor = 1
-        self.N_at_this_zoom = 0
-        self.last_shift_at_this_zoom = None
-        self.skipped = 0
         self.parameters.autolock_failed.value = False
         self.parameters.autolock_locked.value = False
         self.parameters.autolock_watching.value = False
 
+        if self.approacher:
+            self.approacher.reset_properties()
+
     def run(self, x0, x1, spectrum, should_watch_lock=False, auto_offset=True):
         """Starts the autolock.
 
@@ -44,16 +46,19 @@ class Autolock:
         self.parameters.autolock_approaching.value = True
         self.record_first_error_signal(spectrum)
 
-        self.initial_ramp_amplitude = self.parameters.ramp_amplitude.value
-        self.parameters.autolock_initial_ramp_amplitude.value = self.initial_ramp_amplitude
+        self.initial_ramp_speed = self.parameters.ramp_speed.value
+        self.parameters.autolock_initial_ramp_amplitude.value = self.parameters.ramp_amplitude.value
         self.initial_ramp_center = self.parameters.center.value
 
         self.add_data_listener()
 
     def add_data_listener(self):
-        self.parameters.to_plot.change(self.react_to_new_spectrum)
+        if not self._data_listener_added:
+            self.parameters.to_plot.change(self.react_to_new_spectrum)
+            self._data_listener_added = True
 
     def remove_data_listener(self):
+        self._data_listener_added = False
         self.parameters.to_plot.remove_listener(self.react_to_new_spectrum)
 
     def react_to_new_spectrum(self, plot_data):
@@ -82,7 +87,14 @@ class Autolock:
         if plot_data is None:
             return
 
-        if self.parameters.lock.value:
+        is_locked = self.parameters.lock.value
+
+        # check that `plot_data` contains the information we need
+        # otherwise skip this round
+        if not check_plot_data(is_locked, plot_data):
+            return
+
+        if is_locked:
             error_signal = plot_data['error_signal']
             control_signal = plot_data['control_signal']
         else:
@@ -97,35 +109,33 @@ class Autolock:
                 # we have already recorded a spectrum and are now approaching
                 # the line by decreasing the scan range and adapting the
                 # center current multiple times.
-
-                if self.skipped < 3:
-                    # after every step, we skip some data in order to let
-                    # the laser equilibrate
-                    self.skipped += 1
-                    return
-
-                self.skipped = 0
-
-                return self.approach_line(combined_error_signal)
+                if self.approacher is None:
+                    self.approacher = Approacher(
+                        self.control, self.parameters, self.first_error_signal,
+                        self.target_zoom, allow_ramp_speed_change=True
+                    )
+                approaching_finished = self.approacher.approach_line(combined_error_signal)
+                if approaching_finished:
+                    self._lock()
 
             elif self.parameters.autolock_watching.value:
                 # the laser was locked successfully before. Now we check
                 # periodically whether the laser is still in lock
-
                 return self.watch_lock(error_signal, control_signal)
 
             else:
                 # we are done with approaching and have started the lock.
                 # skip some data and check whether we really are in lock
                 # afterwards.
+                return self.after_lock(error_signal, control_signal, plot_data.get('slow'))
 
-                if self.skipped < 3:
-                    self.skipped += 1
-                    return
-
-                self.skipped = 0
-
-                return self.after_lock(control_signal)
+        except SpectrumUncorrelatedException:
+            print('spectrum uncorrelated')
+            if self.parameters.watch_lock.value:
+                print('retry')
+                self.relock()
+            else:
+                self.exposed_stop()
 
         except Exception:
             traceback.print_exc()
@@ -137,8 +147,7 @@ class Autolock:
 
         if self.auto_offset:
             self.parameters.combined_offset.value = -1 * mean_signal
-            # FIXME: disabled due to problem with conversion
-            # rolled_error_signal -= mean_signal
+            rolled_error_signal -= int(mean_signal)
 
         self.parameters.target_slope_rising.value = target_slope_rising
         self.control.exposed_write_data()
@@ -146,51 +155,7 @@ class Autolock:
         self.target_zoom = target_zoom
         self.first_error_signal = rolled_error_signal
 
-    def approach_line(self, error_signal):
-        shift, zoomed_ref, zoomed_err = determine_shift_by_correlation(
-            self.zoom_factor, self.first_error_signal, error_signal
-        )
-        shift *= self.initial_ramp_amplitude
-        self.history.append((zoomed_ref, zoomed_err))
-
-        self.control.exposed_write_data()
-        self.history.append('shift %f' % (-1 * shift))
-
-        self.parameters.center.value -= shift
-        self.control.exposed_write_data()
-
-        if self.N_at_this_zoom > 50:
-            raise Exception('max number of N_at_this_zoom exceeded: %d' % self.N_at_this_zoom)
-
-        if self.last_shift_at_this_zoom is not None:
-            # check that the drift is slow
-            # this is needed for systems that only react slowly to changes in
-            # input parameters. In this case, we have to wait until the reaction
-            # to the last input is done.
-            drift_is_slow = (np.abs(shift - self.last_shift_at_this_zoom)) \
-                            < (0.05 * self.parameters.ramp_amplitude.value)
-            if drift_is_slow:
-                self.N_at_this_zoom = 0
-                self.last_shift_at_this_zoom = None
-
-                zoom_step = 2
-                self.zoom_factor *= zoom_step
-
-                self.control.pause_acquisition()
-
-                self.parameters.ramp_amplitude.value /= zoom_step
-                self.control.exposed_write_data()
-
-                if self.zoom_factor >= self.target_zoom:
-                    self.parameters.autolock_approaching.value = False
-                    self.control.exposed_start_lock()
-
-                self.control.continue_acquisition()
-
-        self.N_at_this_zoom += 1
-        self.last_shift_at_this_zoom = shift
-
-    def after_lock(self, control_signal):
+    def after_lock(self, error_signal, control_signal, slow_out):
         """After locking, this method checks whether the laser really is locked.
 
         If desired, it automatically tries to relock if locking failed, or
@@ -199,28 +164,42 @@ class Autolock:
         def check_whether_in_lock(control_signal):
             """
             The laser is considered in lock if the mean value of the control
-            signal is within the boundaries of the smalles current ramp we had
+            signal is within the boundaries of the smallest current ramp we had
             before turning on the lock.
             """
-            mean = np.mean(control_signal) / 8192
             center = self.parameters.center.value
-            ampl = self.parameters.ramp_amplitude.value
-            return (center - ampl) <= mean <= (center + ampl)
+            initial_ampl = self.parameters.autolock_initial_ramp_amplitude.value
+            ampl = initial_ampl / self.target_zoom
+
+            slow_ramp = self.parameters.sweep_channel.value == ANALOG_OUT0
+            slow_pid = self.parameters.pid_on_slow_enabled.value
+
+            if not slow_ramp and not slow_pid:
+                mean = np.mean(control_signal) / 8192
+                return (center - ampl) <= mean <= (center + ampl)
+            else:
+                if slow_pid and not slow_ramp:
+                    # we cannot handle this case. Just assume the laser is locked.
+                    return True
+
+                return (center - ampl) <= slow_out / 8192 <= (center + ampl)
 
         self.parameters.autolock_locked.value = check_whether_in_lock(control_signal)
 
         if self.parameters.autolock_locked.value and self.should_watch_lock:
             # we start watching the lock status from now on.
             # this is done in `react_to_new_spectrum()` which is called regularly.
+            self.watcher_last_value = np.mean(control_signal) / 8192
             self.parameters.autolock_watching.value = True
         else:
             self.remove_data_listener()
 
             if not self.parameters.autolock_locked.value:
                 if self.should_watch_lock:
+                    self.parameters.autolock_retrying.value = True
                     return self.relock()
 
-                self.control.exposed_reset()
+                self._reset_scan()
                 self.parameters.autolock_failed.value = True
                 self.remove_data_listener()
 
@@ -228,28 +207,29 @@ class Autolock:
 
     def watch_lock(self, error_signal, control_signal):
         """Check whether the laser is still in lock and init a relock if not."""
-        mean = np.abs(np.mean(control_signal) / 8192)
-        still_in_lock = mean < 0.9
+        mean = np.mean(control_signal) / 8192
 
-        if not still_in_lock:
+        diff = np.abs(mean - self.watcher_last_value)
+        lock_lost = diff > self.parameters.watch_lock_threshold.value
+
+        too_close_to_edge = np.abs(mean) > 0.95
+
+        if too_close_to_edge or lock_lost:
+            self.parameters.autolock_retrying.value = True
             self.relock()
 
+        self.watcher_last_value = mean
+
     def relock(self):
         """
         Relock the laser using the reference spectrum recorded in the first
         locking approach.
         """
-        self.reset_properties()
-        self.control.pause_acquisition()
-
         self.parameters.autolock_running.value = True
         self.parameters.autolock_approaching.value = True
 
-        self.parameters.center.value = self.initial_ramp_center
-        self.parameters.ramp_amplitude.value = self.initial_ramp_amplitude
-        self.control.exposed_start_ramp()
-
-        self.control.continue_acquisition()
+        self.reset_properties()
+        self._reset_scan()
 
         # add a listener that listens for new spectrum data and consequently
         # tries to relock.
@@ -264,4 +244,24 @@ class Autolock:
         self.parameters.autolock_watching.value = False
         self.remove_data_listener()
 
-        self.control.exposed_reset()
\ No newline at end of file
+        self._reset_scan()
+        self.parameters.task.value = None
+
+    def _lock(self):
+        self.control.pause_acquisition()
+
+        self.parameters.autolock_approaching.value = False
+
+        self.control.exposed_start_lock()
+
+        self.control.continue_acquisition()
+
+    def _reset_scan(self):
+        self.control.pause_acquisition()
+
+        self.parameters.center.value = self.initial_ramp_center
+        self.parameters.ramp_amplitude.value = self.parameters.autolock_initial_ramp_amplitude.value
+        self.parameters.ramp_speed.value = self.initial_ramp_speed
+        self.control.exposed_start_ramp()
+
+        self.control.continue_acquisition()
\ No newline at end of file
